Option Explicit On
Option Strict On
Imports System.Runtime.InteropServices.ComTypes
Imports System.Runtime.InteropServices
Imports System.Diagnostics
Imports System.Reflection
Imports System.Text
Imports System

Public Class Sample

    Public Shared Sub Main()
        Dim fileName As String = "c:\sample.lnk"

        Dim target As String
        Using link As New LinkItem(fileName)
            target = link.GetTarget()
        End Using

        Console.WriteLine(target)
    End Sub

End Class

<DebuggerStepThrough()> Public Class LinkItem
    Implements IDisposable

    Private Shared ReadOnly typeCShellLink As Type
    Private Shared ReadOnly IID_IShellLinkA As New Guid("000214EE-0000-0000-C000-000000000046")
    Private Shared ReadOnly IID_IShellLinkW As New Guid("000214F9-0000-0000-C000-000000000046")
    Private Shared ReadOnly IID_IShellLink As Guid

    <Flags()> Public Enum SLGP As UInteger
        None = &H0UI
        ShortPath = &H1UI   'Z`i8.3`j̃t@C擾
        UNCPriority = &H2UI 'UNCpX擾
        RawPath = &H4UI     'ϐϊɎ擾
    End Enum

    Private Const MAX_PATH As Integer = 260
    Private Const INFOTIPSIZE As Integer = 1024

    Private Const IShellLink_GetPath As Long = 3
    <UnmanagedFunctionPointer(CallingConvention.StdCall, CharSet:=CharSet.Auto)> _
    Private Delegate Function GetPathDelegate( _
        ByVal thisPtr As IntPtr, ByVal pszFile As StringBuilder, _
        ByVal cchMaxPath As Integer, ByVal pfd As IntPtr, ByVal fFlags As SLGP) As Integer

    Private Const IShellLink_GetArguments As Long = 10
    <UnmanagedFunctionPointer(CallingConvention.StdCall, CharSet:=CharSet.Auto)> _
    Private Delegate Function GetArgumentsDelegate( _
        ByVal thisPtr As IntPtr, ByVal pszArgs As StringBuilder, _
        ByVal cchMaxPath As Integer) As Integer

    Private oShellLink As Object
    Private pShellLink As IntPtr
    Private vtblIShellLink As IntPtr
    Private OnGetPath As GetPathDelegate
    Private OnGetArguments As GetArgumentsDelegate

    Shared Sub New()
        typeCShellLink = Type.GetTypeFromCLSID(New Guid("00021401-0000-0000-C000-000000000046"))
        If Environment.OSVersion.Platform = PlatformID.Win32NT Then
            IID_IShellLink = IID_IShellLinkW
        Else
            IID_IShellLink = IID_IShellLinkA
        End If
    End Sub

    Public Sub New(ByVal fileName As String)
        Try
            oShellLink = Activator.CreateInstance(typeCShellLink)
            DirectCast(oShellLink, IPersistFile).Load(fileName, 0)
            Marshal.QueryInterface(Marshal.GetIUnknownForObject(oShellLink), IID_IShellLink, pShellLink)
            vtblIShellLink = Marshal.ReadIntPtr(pShellLink)

            Dim p As Long = vtblIShellLink.ToInt64()
            Dim pGetPath As IntPtr = Marshal.ReadIntPtr(New IntPtr(p + IShellLink_GetPath * IntPtr.Size))
            Dim pGetArguments As IntPtr = Marshal.ReadIntPtr(New IntPtr(p + IShellLink_GetArguments * IntPtr.Size))
            OnGetPath = DirectCast(Marshal.GetDelegateForFunctionPointer(pGetPath, GetType(GetPathDelegate)), GetPathDelegate)
            OnGetArguments = DirectCast(Marshal.GetDelegateForFunctionPointer(pGetArguments, GetType(GetArgumentsDelegate)), GetArgumentsDelegate)
        Catch
            Dispose(False)
            Throw
        End Try
    End Sub

    Public Function GetTarget() As String
        Dim args As String = GetArguments()
        If args = "" Then
            Return GetLinkPath()
        Else
            Return GetLinkPath() & " " & args
        End If
    End Function

    Public Function GetLinkPath() As String
        Return GetLinkPath(SLGP.None)
    End Function

    Public Function GetLinkPath(ByVal mode As SLGP) As String
        Dim targetPath As New StringBuilder(MAX_PATH, MAX_PATH)
        OnGetPath.Invoke(pShellLink, targetPath, MAX_PATH, IntPtr.Zero, mode)
        Return targetPath.ToString()
    End Function

    Public Function GetArguments() As String
        Dim arguments As New StringBuilder(INFOTIPSIZE, INFOTIPSIZE)
        OnGetArguments.Invoke(pShellLink, arguments, INFOTIPSIZE)
        Return arguments.ToString()
    End Function

    Private disposedValue As Boolean = False

    Protected Overridable Sub Dispose(ByVal disposing As Boolean)
        If Not Me.disposedValue Then
            If disposing Then
            End If
            If pShellLink <> IntPtr.Zero Then
                Marshal.Release(pShellLink)
            End If
            If oShellLink IsNot Nothing And Marshal.IsComObject(oShellLink) Then
                Marshal.ReleaseComObject(oShellLink)
            End If
        End If
        Me.disposedValue = True
    End Sub

    Public Sub Dispose() Implements IDisposable.Dispose
        Dispose(True)
        GC.SuppressFinalize(Me)
    End Sub
End Class
